# -*- coding: utf-8 -*-

#  PycaWM
#  clientunderpointer.py
#  Copyright (c) 2007 Vincent Rasneur, Anaël Verrier

#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; version 3 only.

#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import sys

if sys.version_info < (2, 5):
    raise ImportError('clientunderpointer module needs at least Python 2.5!')

import time

from Xlib import X

from hookmanager import add_pre_hook, remove_pre_hook

class ClientUnderPointer(object):
    def __init__(self, wm, client_fun):
        self.wm = wm
        self.client_fun = client_fun
        self.display = wm.display
        self.screen = self.display.screen()
        self.root = self.screen.root
        self.window = self.root.create_window(0, 0, 1, 1,
                                              1, self.screen.root_depth,
                                              window_class=X.InputOutput,
                                              event_mask=X.ButtonPressMask)
        add_pre_hook(wm.button_press_reply, self.button_press_reply)
        self.window.map()
        if (self.window.grab_pointer(False,
                                     X.ButtonPressMask | X.ButtonReleaseMask |
                                     X.PointerMotionMask,
                                     X.GrabModeAsync,
                                     X.GrabModeAsync,
                                     X.NONE,
                                     X.NONE,
                                     X.CurrentTime) != X.GrabSuccess):
            print 'cannot grab the pointer for the MoveUnderPointer plugin!'
            self.destroy()
            return

    def button_press_reply(self, _, event):
        client = self.wm.client_at_point(event.root_x, event.root_y)
        if client is not None:
            self.client_fun(client)
        self.destroy()

    def destroy(self):
        self.display.ungrab_pointer(X.CurrentTime)
        remove_pre_hook(self.wm.button_press_reply, self.button_press_reply)
        self.window.unmap()
        self.window.destroy()
